@@ -60,6 +60,8 @@ class ZMApi (Base):
         self.access_token_expires = None
         self.refresh_token_expires = None
         self.refresh_token_datetime = None
+        self.access_token_datetime = None
+
         self.legacy_credentials = None
         self.session = requests.Session()
         if options.get('disable_ssl_cert_check'):
@@ -119,6 +121,19 @@ class ZMApi (Base):
         """
         return self.authenticated
 
+
+    # called in _make_request to avoid 401s if possible
+    def _refresh_tokens_if_needed(self):
+        if not (self.access_token_expires and self.refresh_token_expires):
+            return
+        tr = (self.access_token_datetime - datetime.datetime.now()).total_seconds()
+        if (tr >= 60*5): # 5 mins grace 
+            self.logger.Debug(3, 'No need to relogin as access token still has {} minutes remaining'.format(tr/60))
+            return
+        else:
+            self._relogin()
+              
+
     def _relogin(self):
         """ Used for 401. I could use _login too but decided to do a simpler fn
         """
@@ -129,7 +144,7 @@ class ZMApi (Base):
 
 
             if (tr >= 60 * 5): # 5 mins grace
-                self.logger.Debug(1, 'Going to use refresh token as it still has {} minutes remaining'.format(tr/60))
+                self.logger.Debug(2, 'Going to use refresh token as it still has {} minutes remaining'.format(tr/60))
                 self.options['token'] = self.refresh_token
             else:
                 self.logger.Debug (1, 'Refresh token only has {}s of lifetime, going to use user/pass'.format(tr))
@@ -184,7 +199,9 @@ class ZMApi (Base):
                     if rj.get('refresh_token'):
                         self.refresh_token = rj.get('refresh_token')
                     if (rj.get('access_token_expires')):
-                     self.access_token_expires = int(rj.get('access_token_expires'))
+                        self.access_token_expires = int(rj.get('access_token_expires'))
+                        self.access_token_datetime = datetime.datetime.now() + datetime.timedelta(seconds = self.access_token_expires)
+                        self.logger.Debug (1, 'Access token expires on:{} [{}s]'.format(self.access_token_datetime, self.access_token_expires))
                     if (rj.get('refresh_token_expires')):
                         self.refresh_token_expires = int(rj.get('refresh_token_expires'))
                         self.refresh_token_datetime = datetime.datetime.now() + datetime.timedelta(seconds = self.refresh_token_expires)
@@ -228,6 +245,8 @@ class ZMApi (Base):
 
 
     def _make_request(self, url=None, query={}, payload={}, type='get', reauth=True):
+
+        self._refresh_tokens_if_needed()
         type = type.lower()
         if self.auth_enabled:
             if self._versiontuple(self.api_version) >= self._versiontuple('2.0'):
@@ -265,7 +284,6 @@ class ZMApi (Base):
             # Empty response, e.g. to DELETE requests, can't be parsed to json
             # even if the content-type says it is application/json
 
-            
             if r.headers.get('content-type').startswith("application/json") and r.text:
                 return r.json()
             elif r.headers.get('content-type').startswith('image/'):
@@ -273,7 +291,9 @@ class ZMApi (Base):
             else:
                 # A non 0 byte response will usually mean its an image eid request that needs re-login
                 if r.headers.get('content-length') != '0':
-                    raise ValueError ("Non zero response that is not an image or json: {}".format(r.headers.get('content-type')))
+                    raise ValueError ("RELOGIN")
+                else:
+                    raise ValueError ("EMPTY_IMAGE")
                 #return r.text
 
         except requests.exceptions.HTTPError as err:
@@ -284,16 +304,20 @@ class ZMApi (Base):
                 self.logger.Debug (1,'Retrying failed request again...')
                 return self._make_request(url, query, payload, type, reauth=False)
         except ValueError as err:
-            if reauth:
-                self.logger.Debug(1, 'Got ValueError access error: {}'.format(err))
-                self.logger.Debug (1, 'Retrying login once')
-                self._relogin()
-                self.logger.Debug (1,'Retrying failed request again...')
-                return self._make_request(url, query, payload, type, reauth=False)
-            else:
-                raise err
-        else:
-            raise err
+            err_msg = '{}'.format(err)
+            if err_msg == "RELOGIN":
+                if reauth:
+                    self.logger.Debug(1, 'Got ValueError access error: {}'.format(err))
+                    self.logger.Debug (1, 'Retrying login once')
+                    self._relogin()
+                    self.logger.Debug (1,'Retrying failed request again...')
+                    return self._make_request(url, query, payload, type, reauth=False)
+                else:
+                    raise err
+            elif err_msg == "EMPTY_IMAGE":
+                raise ValueError ("Likely bad image index")
+        
+       
 
 
     def monitors(self, options={}):
